<?php

namespace Drupal\xtc\PluginManager\XtcHandler;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Plugin\PluginInspectionInterface;

/**
 * Base class for xtc_handler plugins.
 */
abstract class XtcHandlerPluginBase extends PluginBase
  implements XtcHandlerInterface, PluginInspectionInterface
{

  /**
   * @var string
   */
  protected $type;

  /**
   * @var string
   */
  protected $method;

  /**
   * @var string
   */
  protected $param;

  /**
   * @var mixed
   */
  protected $content;

  /**
   * @var array
   */
  protected $profile = [];

  /**
   * @var array
   */
  protected $options = [];

  /**
   * XtcHandlerPluginBase constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->type = $this->type();
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function process() {
    $this->initProcess();
    $this->runProcess();
    $this->adaptContent();
    return $this;
  }

  protected function initProcess(){
  }

  protected function runProcess(){
  }

  protected function adaptContent(){
  }

  /**
   * @param array $options
   *
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function setOptions($options = []) : XtcHandlerPluginBase {
    $this->options = $options;
    return $this;
  }

  /**
   * @return array
   */
  public function getOptions(){
    return $this->options;
  }

  /**
   * @return mixed|null
   */
  public function content() {
    return $this->content ?? null;
  }

  /**
   * @return mixed|null
   */
  public function values() {
    return $this->content['values'] ?? null;
  }

  /**
   * @return mixed|null
   */
  public function processContent() {
    return $this->process()
                ->content();
  }

  /**
   * @return mixed|null
   */
  public function processValues() {
    return $this->process()
                ->values();
  }

  /**
   * @param $content
   *
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function writeContent($content) {
    return $this->setContent($content)
      ->process();
  }

  /**
   * @param $content
   *
   * @return $this
   */
  public function setContent($content){
    $this->content = $content;
    return $this;
  }

  /**
   * @param array $profile
   *
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function setProfile(array $profile) : XtcHandlerPluginBase{
    $this->profile = $profile;
    return $this;
  }

  /**
   * @param        $method
   * @param string $param
   *
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function init($method, $param = '') : XtcHandlerPluginBase{
    $this->method = $method;
    $this->param = $param;
    return $this;
  }

  /**
   * @return string
   */
  public static function type(){
    return 'other';
  }

}
